/* DO NOT EDIT THIS FILE - it is machine generated */

#include <jni.h>

#ifndef __gnu_xml_libxmlj_dom_GnomeDocumentBuilder__
#define __gnu_xml_libxmlj_dom_GnomeDocumentBuilder__

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT jobject JNICALL Java_gnu_xml_libxmlj_dom_GnomeDocumentBuilder_parseStream (JNIEnv *env, jobject, jobject, jbyteArray, jstring, jstring, jstring, jboolean, jboolean, jboolean, jboolean, jboolean);
JNIEXPORT jobject JNICALL Java_gnu_xml_libxmlj_dom_GnomeDocumentBuilder_createDocument (JNIEnv *env, jobject, jstring, jstring, jobject);

#ifdef __cplusplus
}
#endif

#endif /* __gnu_xml_libxmlj_dom_GnomeDocumentBuilder__ */
