/* DO NOT EDIT THIS FILE - it is machine generated */

#include <jni.h>

#ifndef __gnu_xml_libxmlj_dom_GnomeXPathNodeList__
#define __gnu_xml_libxmlj_dom_GnomeXPathNodeList__

#ifdef __cplusplus
extern "C"
{
#endif

JNIEXPORT void JNICALL Java_gnu_xml_libxmlj_dom_GnomeXPathNodeList_free (JNIEnv *env, jobject, jobject);
JNIEXPORT jint JNICALL Java_gnu_xml_libxmlj_dom_GnomeXPathNodeList_getLength (JNIEnv *env, jobject);
JNIEXPORT jobject JNICALL Java_gnu_xml_libxmlj_dom_GnomeXPathNodeList_item (JNIEnv *env, jobject, jint);

#ifdef __cplusplus
}
#endif

#endif /* __gnu_xml_libxmlj_dom_GnomeXPathNodeList__ */
